<?php
/**
 * SkeletonCart
 * (c) 2017 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace MTDBConnector;

use App\ItemList;
use ORM;

class MTDBItemList implements ItemList
{
    const ENTRY_STATUS_COLUMN = 'entry_status';
    const RELEASE = 2;
    const ACTIVE_COLUMN = 'sc_item_active';
    const ACTIVE = 1;

    protected $dbConfig;
    protected $useEntryID;
    protected $tableName = 'mt_sc_item';
    protected $columns;

    public function __construct($mtConfig, $useEntryID)
    {
        $this->dbConfig = $mtConfig;
        $this->useEntryID = $useEntryID;

        $this->columns = array(
            $this->itemIDColumn() => 'id',
            'sc_item_name' => 'name',
            'sc_item_price' => 'price',
            'sc_item_group_id' => 'groupId',
            'sc_item_stock_quantity' => 'stock',
            'sc_item_weight' => 'weight',
            'sc_item_option_field_1' => 'option1',
            'sc_item_option_field_2' => 'option2',
            'sc_item_option_field_3' => 'option3',
            'sc_item_option_field_4' => 'option4',
            'sc_item_option_field_5' => 'option5',
            'sc_item_blog_id' => 'blogId',
        );
    }

    public function findOne($id)
    {
        $entry = ORM::for_table($this->tableName)
                    ->where(self::ACTIVE_COLUMN, self::ACTIVE)
                    ->where($this->itemIDColumn(), $id)
                    ->find_one();
        if ($entry === false) {
            return false;
        }

        $data = array();
        foreach (array_keys($this->columns) as $column) {
            $data[] = $entry->get($column);
        }

        $nullCount = array_reduce($data, function ($carry, $v) {
            return (is_null($v) || strlen($v) === 0) ? $carry + 1 : $carry;
        }, 0);
        if ($nullCount === count($this->columns)) {
            return false;
        }

        return $data;
    }

    public function itemIDColumn()
    {
        return $this->useEntryID ? 'sc_item_entry_id' : 'sc_item_item_id';
    }

    public function props2columns($props)
    {
        $columns = array();
        foreach ($this->columns as $column => $prop) {
            $columns[$column] = $props[$prop];
        }
        return $columns;
    }

    public function update($props)
    {
        $id = $props['id'];
        $entry = ORM::for_table($this->tableName)->where($this->itemIDColumn(), $id)->find_one();
        if ($entry !== false) {
            // $columns = $this->props2columns($props);
            $columns = array('sc_item_stock_quantity' => $props['stock']);
            $entry->set($columns);
            $entry->save();
            return true;
        }
        return false;
    }

    public function close()
    {
    }

    public function save()
    {
    }

    public function lock()
    {
        $status = ORM::raw_execute("LOCK TABLES {$this->tableName} WRITE");
//		\App::logger()->debug(__METHOD__ . ": lock, status=$status");
        if ($status === false) {
            throw new \RuntimeException('テーブルのロックを取得できませんでした。');
        }
        return $status;
    }

    public function unlock()
    {
        $status = ORM::raw_execute('UNLOCK TABLES');
//		\App::logger()->debug(__METHOD__ . ": unlock, status=$status");
        if ($status === false) {
            throw new \RuntimeException('テーブルのロックを解除できませんでした。');
        }
        return $status;
    }
}
